package autoradio

import (
	"flag"
	"io/ioutil"
	"log"
	"net"
	"strings"

	"git.autistici.org/ale/autoradio/Godeps/_workspace/src/github.com/coreos/go-etcd/etcd"
)

var (
	etcdMachines = flag.String("etcd-servers", "localhost:2379", "Etcd servers (comma-separated list)")
	etcdCertFile = flag.String("etcd-cert", "", "SSL certificate for etcd client")
	etcdKeyFile  = flag.String("etcd-key", "", "SSL private key for etcd client")
	etcdCaFile   = flag.String("etcd-ca", "", "SSL CA certificate for etcd client")
)

func mustLoadFile(path string) string {
	data, err := ioutil.ReadFile(path)
	if err != nil {
		log.Fatal(err)
	}
	return string(data)
}

// Resolve a list of etcd host:port specs, returning URLs.
func resolveAll(input []string, proto string) []string {
	var result []string
	for _, hostport := range input {
		host, port, err := net.SplitHostPort(hostport)
		if err != nil {
			log.Fatalf("Error parsing etcd server spec '%s': %s", hostport, err)
		}
		addrs, err := net.LookupHost(host)
		if err != nil {
			log.Fatalf("Error resolving etcd server spec '%s': %s", hostport, err)
		}
		for _, a := range addrs {
			url := proto + "://" + net.JoinHostPort(a, port)
			result = append(result, url)
		}
	}
	return result
}

// NewEtcdClient creates a new etcd client. It uses command-line flags
// to find servers and connection parameters. If the 'strongReads'
// argument is true, read requests will also be sent to the etcd
// master and should be able to see the effect of previous writes.
func NewEtcdClient(strongReads bool) EtcdClient {
	proto := "http"
	if *etcdCertFile != "" && *etcdKeyFile != "" {
		proto = "https"
	}

	// Resolve etcd servers.
	machines := resolveAll(strings.Split(*etcdMachines, ","), proto)
	if len(machines) == 0 {
		log.Fatal("No etcd servers specified!")
	}

	// Create the etcd client.
	var c *etcd.Client
	if proto == "https" {
		var err error
		c, err = etcd.NewTLSClient(machines,
			mustLoadFile(*etcdCertFile),
			mustLoadFile(*etcdKeyFile),
			mustLoadFile(*etcdCaFile))
		if err != nil {
			log.Fatalf("Error setting up SSL for etcd client: %s", err)
		}
	} else {
		c = etcd.NewClient(machines)
	}

	if strongReads {
		c.SetConsistency(etcd.STRONG_CONSISTENCY)
	} else {
		c.SetConsistency(etcd.WEAK_CONSISTENCY)
	}

	return c
}

// EtcdClient is the etcd client interface used by autoradio. Used to
// decouple our code from the actual etcd API, for testing purposes.
type EtcdClient interface {
	AddChild(string, string, uint64) (*etcd.Response, error)
	Create(string, string, uint64) (*etcd.Response, error)
	CreateDir(string, uint64) (*etcd.Response, error)
	CompareAndSwap(string, string, uint64, string, uint64) (*etcd.Response, error)
	Delete(string, bool) (*etcd.Response, error)
	Get(string, bool, bool) (*etcd.Response, error)
	GetCluster() []string
	Set(string, string, uint64) (*etcd.Response, error)
	SyncCluster() bool
	Update(string, string, uint64) (*etcd.Response, error)
	Watch(string, uint64, bool, chan *etcd.Response, chan bool) (*etcd.Response, error)
}
